@@ -17,7 +17,7 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Callable
+from collections.abc import Callable, Iterable
 from datetime import timedelta
 import logging
 import random
@@ -85,7 +85,10 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
-DISCOVER_TIMEOUT = 30
+# The discover timeout is how long we will wait for and advertisement to be
+# received. If we don't get it in this time we will try again later since
+# the scanner is always running anyways.
+DISCOVER_TIMEOUT = 10
 
 # Battery powered devices may not broadcast once paired until
 # there is an event so we use a long availablity interval.
@@ -93,6 +96,7 @@ AVAILABILITY_INTERVAL = 86400 * 7  # 7 days
 
 NEVER_TIME = -AVAILABILITY_INTERVAL
 
+START_NOTIFY_DEBOUNCE = 1.0
 
 SERVICE_INSTANCE_ID = "E604E95D-A759-4817-87D3-AA005083A0D1"
 SERVICE_INSTANCE_ID_UUID = UUID(SERVICE_INSTANCE_ID)
@@ -105,12 +109,15 @@ SKIP_SYNC_SERVICES = {
 }
 # These characteristics are not readable unless there has been a write
 WRITE_FIRST_REQUIRED_CHARACTERISTICS = {
-    "00000131-0000-1000-8000-0026BB765291",  # Setup Data Stream Transport
-    "00000117-0000-1000-8000-0026BB765291",  # Selected RTP Stream Configuration
-    "00000118-0000-1000-8000-0026BB765291",  # Setup Endpoints
+    CharacteristicsTypes.SETUP_DATA_STREAM_TRANSPORT,  # Setup Data Stream Transport
+    CharacteristicsTypes.SELECTED_RTP_STREAM_CONFIGURATION,  # Selected RTP Stream Configuration
+    CharacteristicsTypes.SETUP_ENDPOINTS,  # Setup Endpoints
     "00000138-0000-1000-8000-0026BB765291",  # Unknown write first characteristic
     "246912DC-8FA3-82ED-DEA4-9EB91D8FC2EE",  # Unknown Vendor char seen on Belkin Wemo Switch
 }
+IGNORE_READ_CHARACTERISTICS = {
+    CharacteristicsTypes.SERVICE_SIGNATURE
+} | WRITE_FIRST_REQUIRED_CHARACTERISTICS
 BLE_AID = 1  # The aid for BLE devices is always 1
 
 ENABLE_BROADCAST_PAYLOAD = TLV.encode_list(
@@ -209,6 +216,10 @@ class BlePairing(AbstractPairing):
         self._decryption_key: DecryptionKey | None = None
         self._broadcast_decryption_key: BroadcastDecryptionKey | None = None
 
+        cached_key = self.broadcast_key
+        if cached_key:
+            self._broadcast_decryption_key = BroadcastDecryptionKey(cached_key)
+
         # Used to keep track of which characteristics we already started
         # notifications for
         self._notifications: set[int] = set()
@@ -232,8 +243,11 @@ class BlePairing(AbstractPairing):
         # Only process disconnected events once
         self._disconnected_events_lock = asyncio.Lock()
 
+        self._start_notify_timer: asyncio.TimerHandle | None = None
+
         self._tried_to_connect_once = False
         self._restore_pending = False
+        self._fetched_gsn_this_session = False
 
     @property
     def address(self) -> str:
@@ -329,10 +343,10 @@ class BlePairing(AbstractPairing):
             logger.debug("%s: Client not connected; rssi=%s", self.name, self.rssi)
             raise AccessoryDisconnectedError(f"{self.name} is not connected")
 
-        if char.handle:
-            endpoint = self.client.get_characteristic_by_handle(char.handle)
-        else:
-            endpoint = self.client.get_characteristic(char.service.type, char.type)
+        endpoint_iid = iid if iid is not None else char.iid
+        endpoint = await self.client.get_characteristic(
+            char.service.type, char.type, endpoint_iid
+        )
 
         pdu_status, result_data = await ble_request(
             self.client,
@@ -340,7 +354,7 @@ class BlePairing(AbstractPairing):
             self._decryption_key,
             opcode,
             endpoint,
-            iid if iid is not None else char.iid,
+            endpoint_iid,
             data,
         )
 
@@ -363,6 +377,7 @@ class BlePairing(AbstractPairing):
         self._notifications = set()
         self._broadcast_notifications = set()
         self._restore_pending = False
+        self._fetched_gsn_this_session = False
 
     async def _ensure_connected(self, attempts: int | None = None) -> bool | None:
         """Ensure that we are connected to the accessory.
@@ -403,10 +418,9 @@ class BlePairing(AbstractPairing):
         char = self.accessories.aid(BLE_AID).characteristics.iid(iid)
 
         # Find the GATT Characteristic object for this iid
-        if char.handle:
-            endpoint = self.client.get_characteristic_by_handle(char.handle)
-        else:
-            endpoint = self.client.get_characteristic(char.service.type, char.type)
+        endpoint = await self.client.get_characteristic(
+            char.service.type, char.type, iid
+        )
 
         # We only want to allow one in flight read
         # and one pending read at a time since there
@@ -429,9 +443,14 @@ class BlePairing(AbstractPairing):
                 # After a char has changed we need to check if the
                 # GSN has changed as well so we don't reconnect
                 # to the accessory if we don't need to
+                if self._fetched_gsn_this_session:
+                    # The spec says we should only do this once per session
+                    # after a characteristic has changed
+                    return
                 protocol_param = await self._get_all_protocol_params()
                 if protocol_param:
                     self.description.state_num = protocol_param.state_number
+                    self._fetched_gsn_this_session = True
 
         def _callback(id: int, data: bytes) -> None:
             logger.debug("%s: Received event for iid=%s: %s", self.name, iid, data)
@@ -512,10 +531,16 @@ class BlePairing(AbstractPairing):
     ) -> ProtocolParams | None:
         accessory_chars = self.accessories.aid(BLE_AID).characteristics
         protocol_param = await self._get_all_protocol_params()
-        await self._get_characteristics_while_connected(
-            [accessory_chars.iid(iid) for _, iid in self.subscriptions],
-            notify_listeners=True,
-        )
+        chars_to_update = []
+        for _, iid in self.subscriptions:
+            char = accessory_chars.iid(iid)
+            if char.broadcast_events or char.disconnected_events:
+                chars_to_update.append(char)
+        if chars_to_update:
+            await self._get_characteristics_while_connected(
+                chars_to_update,
+                notify_listeners=True,
+            )
         return protocol_param
 
     def _async_notification(self, data: HomeKitEncryptedNotification) -> None:
@@ -530,6 +555,13 @@ class BlePairing(AbstractPairing):
             async_create_task(self._process_disconnected_events())
             return
 
+        if not self.description:
+            logger.error(
+                "%s: Received encrypted notification before advertisement.",
+                self.name,
+            )
+            return
+
         start_state_num = self.description.state_num
         # Usually we increment by one, but sometimes we get multiple with the same
         # state number so the first pass is optimistic to reduce the number of
@@ -636,9 +668,13 @@ class BlePairing(AbstractPairing):
                 )
         long_term_pub_key_hex: str = self.pairing_data["iOSDeviceLTPK"]
         long_term_pub_key_bytes = bytes.fromhex(long_term_pub_key_hex)
-        self._broadcast_decryption_key = BroadcastDecryptionKey(
-            self._derive(long_term_pub_key_bytes, b"Broadcast-Encryption-Key")
+        broadcast_key_bytes = self._derive(
+            long_term_pub_key_bytes, b"Broadcast-Encryption-Key"
         )
+        self._broadcast_decryption_key = BroadcastDecryptionKey(broadcast_key_bytes)
+        if self._accessories_state and self.broadcast_key != broadcast_key_bytes:
+            self._accessories_state.broadcast_key = broadcast_key_bytes
+            self._callback_and_save_config_changed(self.config_num)
 
     async def _read_signature(
         self,
@@ -804,7 +840,7 @@ class BlePairing(AbstractPairing):
             ):
                 continue
             for char in service.characteristics:
-                if char.type in WRITE_FIRST_REQUIRED_CHARACTERISTICS:
+                if char.type in IGNORE_READ_CHARACTERISTICS:
                     continue
                 if CharacteristicPermissions.paired_read not in char.perms:
                     continue
@@ -820,7 +856,9 @@ class BlePairing(AbstractPairing):
         for char in chars:
             result = results.get((BLE_AID, char.iid))
             if not result or "value" not in result:
-                logger.debug("%s: No value for %s", self.name, char)
+                logger.debug(
+                    "%s: No value for %s/%s", self.name, char.service.type, char.type
+                )
                 continue
             char.value = result["value"]
 
@@ -913,9 +951,13 @@ class BlePairing(AbstractPairing):
             if self._shutdown:
                 return
 
-            self._tried_to_connect_once = True
-
-            made_connection = await self._ensure_connected(attempts)
+            try:
+                made_connection = await self._ensure_connected(attempts)
+            finally:
+                # Only set _tried_to_connect_once after the connection
+                # attempt is complete so we don't try to process disconnected
+                # events while we are still trying to connect.
+                self._tried_to_connect_once = True
 
             logger.debug(
                 "%s: Populating accessories and characteristics: made_connection=%s restore_pending=%s",
@@ -948,7 +990,9 @@ class BlePairing(AbstractPairing):
                 )
                 accessories = await self._async_fetch_gatt_database()
                 new_config_num = self.description.config_num if self.description else 0
-                self._accessories_state = AccessoriesState(accessories, new_config_num)
+                self._accessories_state = AccessoriesState(
+                    accessories, new_config_num, self.broadcast_key
+                )
                 update_values = True
 
             if not self._encryption_key:
@@ -1020,18 +1064,25 @@ class BlePairing(AbstractPairing):
             self.rssi,
         )
         await self._async_subscribe_broadcast_events(subscriptions)
-        await self._async_start_notify_subscriptions(subscriptions)
         self._restore_pending = False
         # After we have restored subscriptions, we need to read
         # the state number again to make sure we are in sync
         protocol_param = await self._get_all_protocol_params()
         if protocol_param:
             self.description.state_num = protocol_param.state_number
+        self._async_schedule_start_notify_subscriptions()
 
-    async def _async_start_notify_subscriptions(
-        self, subscriptions: list[tuple[int, int]]
-    ) -> None:
-        """Start notifications for the given subscriptions."""
+    async def _async_start_notify_subscriptions(self) -> None:
+        """Start notifications for the given subscriptions.
+
+        If we have an error or are disconnected we do not want
+        to retry, as it usually means the accessory indented
+        to disconnect us and we are now using disconnected
+        events or encrypted broadcasts to get updates.
+        """
+        if not self.client or not self.client.is_connected:
+            return
+        subscriptions = list(self.subscriptions)
         for _, iid in subscriptions:
             if iid in self._notifications:
                 continue
@@ -1142,9 +1193,9 @@ class BlePairing(AbstractPairing):
 
         async with self._ble_request_lock:
             for char in characteristics:
-                if char.type in WRITE_FIRST_REQUIRED_CHARACTERISTICS:
+                if char.type in IGNORE_READ_CHARACTERISTICS:
                     logger.debug(
-                        "%s: Ignoring write first only characteristic %s",
+                        "%s: Ignoring characteristic %s",
                         self.name,
                         char.iid,
                     )
@@ -1160,11 +1211,21 @@ class BlePairing(AbstractPairing):
                     # we need to skip in this case.
                     if ex.status == PDUStatus.INVALID_REQUEST:
                         logger.debug(
-                            "%s: Reading characteristic %s resulted in an invalid request (skipped)",
+                            "%s: Reading characteristic %s with iid %s resulted in an invalid request (skipped)",
                             self.name,
+                            char.iid,
                             char.type,
                         )
                         continue
+                    logger.exception(
+                        "%s: Reading characteristic %s with iid %s resulted in an error: %s",
+                        self.name,
+                        char.type,
+                        char.iid,
+                        ex,
+                    )
+                    continue
+
                 decoded = dict(TLV.decode_bytes(data))[1]
 
                 logger.debug(
@@ -1182,7 +1243,7 @@ class BlePairing(AbstractPairing):
                     logger.debug(
                         "%s: Failed to decode characteristic for %s from %s: %s",
                         self.name,
-                        char,
+                        char.type,
                         decoded,
                         ex,
                     )
@@ -1258,22 +1319,48 @@ class BlePairing(AbstractPairing):
 
         return results
 
-    # No retry since disconnected events are ok as well
-    @operation_lock
-    async def subscribe(self, characteristics):
+    async def subscribe(self, characteristics: Iterable[tuple[int, int]]) -> None:
+        """Subscribe to characteristics."""
         new_chars = await super().subscribe(characteristics)
         if not new_chars or not self.client or not self.client.is_connected:
             # Don't force a new connection if we are not already
             # connected as we will get disconnected events.
             return
+        # We do not want to block setup of the accessory so we
+        # do not wait for the result as any failures will be
+        # handled by the retry logic or fallback to disconnected
+        # events.
+        async_create_task(self._async_subscribe(new_chars))
+        self._async_schedule_start_notify_subscriptions()
+
+    @operation_lock
+    @retry_bluetooth_connection_error()
+    @restore_connection_and_resume
+    async def _async_subscribe(self, new_chars: Iterable[tuple[int, int]]) -> None:
+        """Subscribe to new characteristics."""
         logger.debug("%s: subscribing to %s", self.name, new_chars)
         await self._populate_accessories_and_characteristics()
         if not self._broadcast_decryption_key:
             await self._async_set_broadcast_encryption_key()
         await self._async_subscribe_broadcast_events(new_chars)
-        await self._async_start_notify_subscriptions(new_chars)
 
-    async def unsubscribe(self, characteristics):
+    def _async_schedule_start_notify_subscriptions(self) -> None:
+        """Schedule start notify subscriptions."""
+        if self._start_notify_timer:
+            self._start_notify_timer.cancel()
+            self._start_notify_timer = None
+
+        def _async_start_notify_subscriptions() -> None:
+            """Start notify subscriptions."""
+            self._start_notify_timer = None
+            async_create_task(self._async_start_notify_subscriptions())
+
+        loop = asyncio.get_running_loop()
+        self._start_notify_timer = loop.call_later(
+            START_NOTIFY_DEBOUNCE, _async_start_notify_subscriptions
+        )
+
+    async def unsubscribe(self, characteristics: Iterable[tuple[int, int]]) -> None:
         pass
 
     async def identify(self):
